@@ -1,5 +1,5 @@
 ###########################################################################
-# $Id: dcb61f7c26b07351c6be5ff28d91b0a37066d744 $                 #
+# $Id: 84846b20c93e92ba785a9f9e49375246309b48b9 $                 #
 #                                                                         #
 #  schema.sql                       rlm_sql - FreeRADIUS SQL Module       #
 #                                                                         #
@@ -29,8 +29,8 @@ CREATE TABLE IF NOT EXISTS radacct (
   acctinterval int(12) default NULL,
   acctsessiontime int(12) unsigned default NULL,
   acctauthentic varchar(32) default NULL,
-  connectinfo_start varchar(50) default NULL,
-  connectinfo_stop varchar(50) default NULL,
+  connectinfo_start varchar(128) default NULL,
+  connectinfo_stop varchar(128) default NULL,
   acctinputoctets bigint(20) default NULL,
   acctoutputoctets bigint(20) default NULL,
   calledstationid varchar(50) NOT NULL default '',
@@ -43,6 +43,7 @@ CREATE TABLE IF NOT EXISTS radacct (
   framedipv6prefix varchar(45) NOT NULL default '',
   framedinterfaceid varchar(44) NOT NULL default '',
   delegatedipv6prefix varchar(45) NOT NULL default '',
+  class varchar(64) default NULL,
   PRIMARY KEY (radacctid),
   UNIQUE KEY acctuniqueid (acctuniqueid),
   KEY username (username),
@@ -56,7 +57,8 @@ CREATE TABLE IF NOT EXISTS radacct (
   KEY acctstarttime (acctstarttime),
   KEY acctinterval (acctinterval),
   KEY acctstoptime (acctstoptime),
-  KEY nasipaddress (nasipaddress)
+  KEY nasipaddress (nasipaddress),
+  KEY class (class)
 ) ENGINE = INNODB;
 
 #
@@ -144,8 +146,10 @@ CREATE TABLE IF NOT EXISTS radpostauth (
   pass varchar(64) NOT NULL default '',
   reply varchar(32) NOT NULL default '',
   authdate timestamp(6) NOT NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
+  class varchar(64) default NULL,
   PRIMARY KEY  (id),
-  KEY username (username(32))
+  KEY username (username),
+  KEY class (class)
 ) ENGINE = INNODB;
 
 #
@@ -163,4 +167,13 @@ CREATE TABLE IF NOT EXISTS nas (
   description varchar(200) DEFAULT 'RADIUS Client',
   PRIMARY KEY (id),
   KEY nasname (nasname)
-);
+) ENGINE = INNODB;
+
+#
+# Table structure for table 'nasreload'
+#
+CREATE TABLE IF NOT EXISTS nasreload (
+  nasipaddress varchar(15) NOT NULL,
+  reloadtime datetime NOT NULL,
+  PRIMARY KEY (nasipaddress)
+) ENGINE = INNODB;
