@@ -1,7 +1,7 @@
 /*
  * packet.c	Generic packet manipulation functions.
  *
- * Version:	$Id: 971980b38e8a14767879303059eb8883ace5aae3 $
+ * Version:	$Id: 0f870f514eeb88d5f2ca348ecd9c01c77411abda $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000-2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 971980b38e8a14767879303059eb8883ace5aae3 $")
+RCSID("$Id: 0f870f514eeb88d5f2ca348ecd9c01c77411abda $")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -723,6 +723,8 @@ bool fr_packet_list_id_alloc(fr_packet_list_t *pl, int proto,
 	 */
 
 	id = fd = -1;
+	if (request->id >= 0 && request->id < 256)
+		id = request->id;
 	start_i = fr_rand() & SOCKOFFSET_MASK;
 
 #define ID_i ((i + start_i) & SOCKOFFSET_MASK)
@@ -831,6 +833,18 @@ bool fr_packet_list_id_alloc(fr_packet_list_t *pl, int proto,
 		 *	Otherwise, this socket is OK to use.
 		 */
 
+		/*
+		 *	An explicit ID was requested
+		 */
+
+		if (id != -1) {
+			if  ((ps->id[(id >> 3) & 0x1f] & (1 << (id & 0x07))) != 0) continue;
+
+			ps->id[(id >> 3) & 0x1f] |= (1 << (id & 0x07));
+			fd = i;
+			break;
+		}
+
 		/*
 		 *	Look for a free Id, starting from a random number.
 		 */
@@ -1076,7 +1090,7 @@ void fr_packet_header_print(FILE *fp, RADIUS_PACKET *packet, bool received)
 	 *	This really belongs in a utility library
 	 */
 	if (is_radius_code(packet->code)) {
-		fprintf(fp, "%s %s Id %i from %s%s%s:%x to %s%s%s:%u length %zu\n",
+		fprintf(fp, "%s %s Id %i from %s%s%s:%u to %s%s%s:%u length %zu\n",
 		        received ? "Received" : "Sent",
 		        fr_packet_codes[packet->code],
 		        packet->id,
@@ -1094,7 +1108,7 @@ void fr_packet_header_print(FILE *fp, RADIUS_PACKET *packet, bool received)
 		        packet->dst_port,
 		        packet->data_len);
 	} else {
-		fprintf(fp, "%s code %u Id %i from %s%s%s:%u to %s%s%s:%i length %zu\n",
+		fprintf(fp, "%s code %u Id %i from %s%s%s:%u to %s%s%s:%u length %zu\n",
 		        received ? "Received" : "Sent",
 		        packet->code,
 		        packet->id,
