@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: 8a9297b5d6b6e8f039fa475364b0e1c00fb2c270 $
+ * Version:	$Id: 75bb5d616e1ea7b053e5951e1d55c23920c20400 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,10 +26,11 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 8a9297b5d6b6e8f039fa475364b0e1c00fb2c270 $")
+RCSID("$Id: 75bb5d616e1ea7b053e5951e1d55c23920c20400 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
+#include <freeradius-devel/md5.h>
 #include <freeradius-devel/rad_assert.h>
 
 #ifdef HAVE_DIRENT_H
@@ -353,6 +354,7 @@ error:
 
 	file->filename = filename;
 	file->cs = cs;
+	file->from_dir = from_dir;
 
 	if (fstat(fd, &file->buf) == 0) {
 #ifdef S_IWOTH
@@ -1423,12 +1425,13 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 {
 	int rcode;
 	bool deprecated, required, attribute, secret, file_input, cant_be_empty, tmpl, multi, file_exists;
+	bool ignore_dflt;
 	char **q;
 	char const *value;
 	CONF_PAIR *cp = NULL;
 	fr_ipaddr_t *ipaddr;
-	char buffer[8192];
 	CONF_ITEM *c_item;
+	char buffer[8192];
 
 	if (!cs) {
 		cf_log_err(&(cs->item), "No enclosing section for configuration item \"%s\"", name);
@@ -1446,6 +1449,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	cant_be_empty = (type & PW_TYPE_NOT_EMPTY);
 	tmpl = (type & PW_TYPE_TMPL);
 	multi = (type & PW_TYPE_MULTI);
+	ignore_dflt = (type & PW_TYPE_IGNORE_DEFAULT);
 
 	if (attribute) required = true;
 	if (required) cant_be_empty = true;	/* May want to review this in the future... */
@@ -1469,7 +1473,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	 *	section, use the default value.
 	 */
 	if (!cp) {
-		if (deprecated) return 0;	/* Don't set the default value */
+		if (deprecated || ignore_dflt) return 0;	/* Don't set the default value */
 
 		rcode = 1;
 		value = dflt;
@@ -1500,7 +1504,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 				     next->item.lineno, name);
 			}
 		}
-										   
+
 		if (multi) {
 			while ((next = cf_pair_find_next(cs, next, name)) != NULL) {
 				/*
@@ -1657,6 +1661,62 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 				cf_log_err(&(cs->item),"Failed expanding variable %s", name);
 				return -1;
 			}
+
+		} else if (cf_new_escape && (cp->rhs_type == T_DOUBLE_QUOTED_STRING) && (strchr(value, '\\') != NULL)) {
+			char const *p = value;
+			char *s = buffer;
+			char *end = buffer + sizeof(buffer);
+			unsigned int x;
+
+			/*
+			 *	We pass !cf_new_escape() to gettoken() when we parse the RHS of a CONF_PAIR
+			 *	above.  But gettoken() unescapes the \", and doesn't unescape anything else.
+			 *	So we do it here.
+			 */
+			while (*p && (s < end)) {
+				if (*p != '\\') {
+					*(s++) = *(p++);
+					continue;
+				}
+
+				p++;
+
+				switch (*p) {
+				case 'r':
+					*s++ = '\r';
+					break;
+				case 'n':
+					*s++ = '\n';
+					break;
+				case 't':
+					*s++ = '\t';
+					break;
+
+				default:
+					if (*p >= '0' && *p <= '9' &&
+					    sscanf(p, "%3o", &x) == 1) {
+						if (!x) {
+							cf_log_err(&(cs->item), "Cannot have embedded zeros in value for %s", name);
+							return -1;
+						}
+
+						*s++ = x;
+						p += 2;
+					} else
+						*s++ = *p;
+					break;
+				}
+				p++;
+			}
+
+			if (s == end) {
+				cf_log_err(&(cs->item), "Failed expanding value for %s", name);
+				return -1;
+			}
+
+			*s = '\0';
+
+			value = buffer;
 		}
 
 		if (cant_be_empty && (value[0] == '\0')) goto cant_be_empty;
@@ -1803,6 +1863,7 @@ static void cf_section_parse_init(CONF_SECTION *cs, void *base,
 				  CONF_PARSER const *variables)
 {
 	int i;
+	void *data;
 
 	for (i = 0; variables[i].name != NULL; i++) {
 		if (variables[i].type == PW_TYPE_SUBSECTION) {
@@ -1827,9 +1888,13 @@ static void cf_section_parse_init(CONF_SECTION *cs, void *base,
 				subcs->item.lineno = cs->item.lineno;
 				cf_item_add(cs, &(subcs->item));
 			}
+			if (base) {
+				data = ((uint8_t *)base) + variables[i].offset;
+			} else {
+				data = NULL;
+			}
 
-			cf_section_parse_init(subcs, (uint8_t *)base + variables[i].offset,
-					      (CONF_PARSER const *) variables[i].dflt);
+			cf_section_parse_init(subcs, data, (CONF_PARSER const *) variables[i].dflt);
 			continue;
 		}
 
@@ -1927,8 +1992,13 @@ int cf_section_parse(CONF_SECTION *cs, void *base, CONF_PARSER const *variables)
 				goto finish;
 			}
 
-			ret = cf_section_parse(subcs, (uint8_t *)base + variables[i].offset,
-					       (CONF_PARSER const *) variables[i].dflt);
+			if (base) {
+				data = ((uint8_t *)base) + variables[i].offset;
+			} else {
+				data = NULL;
+			}
+
+			ret = cf_section_parse(subcs, data, (CONF_PARSER const *) variables[i].dflt);
 			if (ret < 0) goto finish;
 			continue;
 		} /* else it's a CONF_PAIR */
@@ -2264,6 +2334,34 @@ static char const *cf_local_file(char const *base, char const *filename,
 	return buffer;
 }
 
+static bool cf_md5_initted = false;
+static FR_MD5_CTX conf_context;
+
+void cf_md5_init(void)
+{
+	fr_md5_init(&conf_context);
+	cf_md5_initted = true;
+}
+
+
+static void cf_md5_update(char const *p)
+{
+	if (!cf_md5_initted) return;
+
+	fr_md5_update(&conf_context, (uint8_t const *)p, strlen(p));
+}
+
+void cf_md5_final(uint8_t *digest)
+{
+	if (!cf_md5_initted) {
+		memset(digest, 0, MD5_DIGEST_LENGTH);
+		return;
+	}
+
+	fr_md5_final(digest, &conf_context);
+	cf_md5_initted = false;
+}
+
 
 /*
  *	Read a part of the config file.
@@ -2300,6 +2398,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 		 *	Get data, and remember if we are at EOF.
 		 */
 		at_eof = (fgets(cbuf, sizeof(buf) - (cbuf - buf), fp) == NULL);
+		cf_md5_update(cbuf);
 		(*lineno)++;
 
 		/*
@@ -2537,6 +2636,7 @@ static int cf_section_read(char const *filename, int *lineno, FILE *fp,
 							value, dp->d_name);
 					if (slen >= (int) sizeof(buf2) || slen < 0) {
 						ERROR("%s: Full file path is too long.", dp->d_name);
+						closedir(dir);
 						return -1;
 					}
 					if ((stat(buf2, &stat_buf) != 0) ||
