@@ -1,7 +1,7 @@
 /*
  * evaluate.c	Evaluate complex conditions
  *
- * Version:	$Id: a8ef5d93db8a4a7429a059471549a9df2501d582 $
+ * Version:	$Id: c8585b67e0e0940e4bc328d425d7b2ca4d5941ad $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2007  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: a8ef5d93db8a4a7429a059471549a9df2501d582 $")
+RCSID("$Id: c8585b67e0e0940e4bc328d425d7b2ca4d5941ad $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -61,7 +61,7 @@ static bool all_digits(char const *string)
 
 	if (*p == '-') p++;
 
-	while (isdigit((int) *p)) p++;
+	while (isdigit((uint8_t) *p)) p++;
 
 	return (*p == '\0');
 }
@@ -173,6 +173,9 @@ static int cond_do_regex(REQUEST *request, fr_cond_t const *c,
 	switch (map->rhs->type) {
 	case TMPL_TYPE_REGEX_STRUCT: /* pre-compiled to a regex */
 		preg = map->rhs->tmpl_preg;
+#ifdef HAVE_PCRE
+		rad_assert(preg->precompiled);
+#endif
 		break;
 
 	default:
@@ -187,6 +190,9 @@ static int cond_do_regex(REQUEST *request, fr_cond_t const *c,
 			return -1;
 		}
 		preg = rreg;
+#ifdef HAVE_PCRE
+		rad_assert(!preg->precompiled);
+#endif
 		break;
 	}
 
@@ -790,7 +796,7 @@ int radius_evaluate_cond(REQUEST *request, int modreturn, int depth, fr_cond_t c
 
 
 /*
- *	The fr_pair_list_move() function in src/lib/valuepair.c does all sorts of
+ *	The fr_pair_list_move() function in src/lib/pair.c does all sorts of
  *	extra magic that we don't want here.
  *
  *	FIXME: integrate this with the code calling it, so that we
@@ -804,6 +810,7 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from, bool d
 	VALUE_PAIR *vp, *next, **last;
 	VALUE_PAIR **from_list, **to_list;
 	VALUE_PAIR *append, **append_tail;
+	VALUE_PAIR *prepend;
 	VALUE_PAIR *to_copy;
 	bool *edited = NULL;
 	REQUEST *fixup = NULL;
@@ -837,6 +844,8 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from, bool d
 	for (vp = fr_cursor_init(&cursor, to); vp; vp = fr_cursor_next(&cursor)) count++;
 	to_list = talloc_array(request, VALUE_PAIR *, count);
 
+	prepend = NULL;
+
 	append = NULL;
 	append_tail = &append;
 
@@ -882,6 +891,20 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from, bool d
 		 */
 		if (from_list[i]->op == T_OP_ADD) goto do_append;
 
+		/*
+		 *	The attribute needs to be prepended to the "to"
+		 *	list - store it in the prepend list
+		 */
+
+		if (from_list[i]->op == T_OP_PREPEND) {
+			RDEBUG4("::: PREPENDING %s FROM %d TO %d",
+			       from_list[i]->da->name, i, tailto);
+			from_list[i]->next = prepend;
+			prepend = from_list[i];
+			prepend->op = T_OP_EQ;
+			from_list[i] = NULL;
+			continue;
+		}
 		found = false;
 		for (j = 0; j < to_count; j++) {
 			if (edited[j] || !to_list[j] || !from_list[i]) continue;
@@ -1038,7 +1061,6 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from, bool d
 	 */
 	for (i = 0; i < from_count; i++) {
 		if (!from_list[i]) continue;
-
 		fr_pair_list_free(&from_list[i]);
 	}
 	talloc_free(from_list);
@@ -1057,6 +1079,17 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from, bool d
 		fixup = request->parent;
 	}
 
+	/*
+	 *	Walk the list of "prepend" attributes first
+	 */
+	for (vp = prepend; vp != NULL; vp = vp->next) {
+		*last = vp;
+		last = &(*last)->next;
+	}
+
+	/*
+	 *	Next add on remaining items in the "to" list
+	 */
 	for (i = 0; i < tailto; i++) {
 		if (!to_list[i]) continue;
 
