@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: 9ccb3103ae0a129b258c2e8c84e92b033144a46f $
+ * Version:	$Id: 8f83690b6b4c322aeb3edee38695f141f29cfed6 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-RCSID("$Id: 9ccb3103ae0a129b258c2e8c84e92b033144a46f $")
+RCSID("$Id: 8f83690b6b4c322aeb3edee38695f141f29cfed6 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -285,7 +285,7 @@ char const *fr_dlerror(void)
 	return dlerror();
 }
 
-static int virtual_server_idx(char const *name)
+static uint32_t virtual_server_idx(char const *name)
 {
 	uint32_t hash;
 
@@ -298,11 +298,11 @@ static int virtual_server_idx(char const *name)
 
 static virtual_server_t *virtual_server_find(char const *name)
 {
-	rlm_rcode_t rcode;
+	uint32_t hash;
 	virtual_server_t *server;
 
-	rcode = virtual_server_idx(name);
-	for (server = virtual_servers[rcode];
+	hash = virtual_server_idx(name);
+	for (server = virtual_servers[hash];
 	     server != NULL;
 	     server = server->next) {
 		if (!name && !server->name) break;
@@ -317,6 +317,7 @@ static virtual_server_t *virtual_server_find(char const *name)
 static int _virtual_server_free(virtual_server_t *server)
 {
 	if (server->components) rbtree_free(server->components);
+
 	return 0;
 }
 
@@ -345,7 +346,7 @@ void virtual_servers_free(time_t when)
 			 */
 			if ((when == 0) ||
 			    ((server->created < when) && server->can_free)) {
-				*last = server->next;
+				*last = next;
 				talloc_free(server);
 			} else {
 				last = &(server->next);
@@ -1252,9 +1253,10 @@ static int load_byserver(CONF_SECTION *cs)
 	bool found;
 	char const *name = cf_section_name2(cs);
 	rbtree_t *components;
-	virtual_server_t *server = NULL;
+	virtual_server_t *server = NULL, *old;
 	indexed_modcallable *c;
 	bool is_bare;
+	uint32_t hash;
 
 	if (name) {
 		cf_log_info(cs, "server %s { # from file %s",
@@ -1466,27 +1468,19 @@ static int load_byserver(CONF_SECTION *cs)
 	}
 
 	/*
-	 *	Now that it is OK, insert it into the list.
+	 *	Mark the existing (i.e. old) server as no longer used.
 	 *
-	 *	This is thread-safe...
+	 *	Do this _before_ adding the new virtual server.
 	 */
-	comp = virtual_server_idx(name);
-	server->next = virtual_servers[comp];
-	virtual_servers[comp] = server;
+	old = virtual_server_find(name);
+	if (old) old->can_free = true;
 
 	/*
-	 *	Mark OLDER ones of the same name as being unused.
+	 *	Insert the new server at the head of the list.
 	 */
-	server = server->next;
-	while (server) {
-		if ((!name && !server->name) ||
-		    (name && server->name &&
-		     (strcmp(server->name, name) == 0))) {
-			server->can_free = true;
-			break;
-		}
-		server = server->next;
-	}
+	hash = virtual_server_idx(name);
+	server->next = virtual_servers[hash];
+	virtual_servers[hash] = server;
 
 	return 0;
 }
