@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: 2959d8278a2a839c3f5aaa047e12b15630127485 $
+ * Version:     $Id: eb81b8a6721f5f35e636802f5c47584620a51444 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2007  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: 2959d8278a2a839c3f5aaa047e12b15630127485 $")
+RCSID("$Id: eb81b8a6721f5f35e636802f5c47584620a51444 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/realms.h>
@@ -403,6 +403,14 @@ static ssize_t xlat_home_server_dynamic(UNUSED void *instance, REQUEST *request,
 	}
 
 	home = home_server_byname(name, type);
+
+	/*
+	 *	If we're looking for an auth / acct homeserver, allow for auth+acct
+	 */
+	if (!home && ((type == HOME_TYPE_AUTH) || (type == HOME_TYPE_ACCT))) {
+		home = home_server_byname(name, HOME_TYPE_AUTH_ACCT);
+	}
+
 	if (!home) {
 		*out = '\0';
 		return 0;
@@ -452,6 +460,12 @@ static CONF_PARSER limit_config[] = {
 	{ "max_requests", FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, limit.max_requests), "0" },
 	{ "lifetime", FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, limit.lifetime), "0" },
 	{ "idle_timeout", FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, limit.idle_timeout), "0" },
+#ifdef SO_RCVTIMEO
+	{ "read_timeout", FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, limit.read_timeout), NULL },
+#endif
+#ifdef SO_SNDTIMEO
+	{ "write_timeout", FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, limit.write_timeout), NULL },
+#endif
 	CONF_PARSER_TERMINATOR
 };
 
@@ -475,8 +489,11 @@ static CONF_PARSER home_server_recv_coa[] = {
 
 #endif
 
+static const char *require_message_authenticator = NULL;
+
 static CONF_PARSER home_server_config[] = {
 	{ "nonblock", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, home_server_t, nonblock), "no" },
+	{ "require_message_authenticator", FR_CONF_POINTER(PW_TYPE_STRING| PW_TYPE_IGNORE_DEFAULT, &require_message_authenticator), NULL },
 	{ "ipaddr", FR_CONF_OFFSET(PW_TYPE_COMBO_IP_ADDR, home_server_t, ipaddr), NULL },
 	{ "ipv4addr", FR_CONF_OFFSET(PW_TYPE_IPV4_ADDR, home_server_t, ipaddr), NULL },
 	{ "ipv6addr", FR_CONF_OFFSET(PW_TYPE_IPV6_ADDR, home_server_t, ipaddr), NULL },
@@ -632,7 +649,7 @@ static bool home_server_insert(home_server_t *home, CONF_SECTION *cs)
 		return false;
 	}
 
-	if (!home->virtual_server && !rbtree_insert(home_servers_byaddr, home)) {
+	if (!home->virtual_server && !home->dynamic && !rbtree_insert(home_servers_byaddr, home)) {
 		rbtree_deletebydata(home_servers_byname, home);
 		cf_log_err_cs(cs, "Internal error %d adding home server %s", __LINE__, home->log_name);
 		return false;
@@ -674,7 +691,7 @@ bool realm_home_server_add(home_server_t *home)
 		return false;
 	}
 
-	if (!home->virtual_server && (rbtree_finddata(home_servers_byaddr, home) != NULL)) {
+	if (!home->virtual_server && !home->dynamic && (rbtree_finddata(home_servers_byaddr, home) != NULL)) {
 		char buffer[INET6_ADDRSTRLEN + 3];
 
 		inet_ntop(home->ipaddr.af, &home->ipaddr.ipaddr, buffer, sizeof(buffer));
@@ -780,6 +797,9 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 	home->cs = cs;
 	home->state = HOME_STATE_UNKNOWN;
 	home->proto = IPPROTO_UDP;
+	home->require_ma = main_config.require_ma;
+
+	require_message_authenticator = false;
 
 	/*
 	 *	Parse the configuration into the home server
@@ -787,6 +807,10 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 	 */
 	if (cf_section_parse(cs, home, home_server_config) < 0) goto error;
 
+	if (fr_bool_auto_parse(cf_pair_find(cs, "require_message_authenticator"), &home->require_ma, require_message_authenticator) < 0) {
+		goto error;
+	}
+
 	/*
 	 *	It has an IP address, it must be a remote server.
 	 */
@@ -954,7 +978,7 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 		home->proto = proto;
 	}
 
-	if (!home->virtual_server && rbtree_finddata(home_servers_byaddr, home)) {
+	if (!home->virtual_server && !home->dynamic && rbtree_finddata(home_servers_byaddr, home)) {
 		cf_log_err_cs(cs, "Duplicate home server");
 		goto error;
 	}
@@ -1116,11 +1140,18 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 		if (tls) {
 			int rcode;
 
+			/*
+			 *	We don't require this for TLS connections.
+			 */
+			home->require_ma = false;
+
 			home->tls = tls_client_conf_parse(tls);
 			if (!home->tls) {
 				goto error;
 			}
 
+			home->tls->name = "RADIUS/TLS";
+
 			/*
 			 *	Connection timeouts for outgoing TLS connections.
 			 */
@@ -2473,6 +2504,11 @@ int realms_init(CONF_SECTION *config)
 	     cs = cf_subsection_find_next(config, cs, "realm")) {
 		if (!realm_add(rc, cs)) {
 		error:
+			/*
+			 *	realm_config is freed by realms_free()
+			 *	so if rc == realm_config set it to NULL here.
+			 */
+			if (rc == realm_config) rc = NULL;
 			realms_free();
 			/*
 			 *	Must be called after realms_free as home_servers
@@ -2530,7 +2566,7 @@ int realms_init(CONF_SECTION *config)
 		dir = opendir(rc->directory);
 		if (!dir) {
 			cf_log_err_cs(config, "Error reading directory %s: %s",
-				      rc->directory, fr_syserror(errno));				      
+				      rc->directory, fr_syserror(errno));
 			goto error;
 		}
 
@@ -2547,7 +2583,7 @@ int realms_init(CONF_SECTION *config)
 			 *	Skip the TLS configuration.
 			 */
 			if (strcmp(dp->d_name, "tls.conf") == 0) continue;
-		
+
 			/*
 			 *	Check for valid characters
 			 */
@@ -3124,11 +3160,12 @@ home_pool_t *home_pool_byname(char const *name, int type)
 	return rbtree_finddata(home_pools_byname, &mypool);
 }
 
+
 int home_server_afrom_file(char const *filename)
 {
 	CONF_SECTION *cs, *subcs;
 	char const *p;
-	home_server_t *home;
+	home_server_t *home, *old;
 
 	if (!realm_config->dynamic) {
 		fr_strerror_printf("Must set \"dynamic = true\" in proxy.conf for dynamic home servers to work");
@@ -3169,19 +3206,27 @@ int home_server_afrom_file(char const *filename)
 
 	home->dynamic = true;
 
+#ifdef WITH_TLS
+	/*
+	 *	All of the other code assumes that only TLS sockets
+	 *	have child listeners.  See listen.c for references to
+	 *	...->listeners, which are all inside of blocks which
+	 *	check for TLS.
+	 */
+	if (!home->tls) {
+		fr_strerror_printf("Dynamic home_server '%s' does not use TLS - ignoring it.", p);
+		talloc_free(home);
+		goto error;
+	}
+#endif
+
 	if (home->virtual_server) {
 		fr_strerror_printf("Dynamic home_server '%s' cannot have 'server = %s' configuration item", p, home->virtual_server);
 		talloc_free(home);
 		goto error;
 	}
 
-	if (home->dual) {
-		fr_strerror_printf("Dynamic home_server '%s' is missing 'type', or it is set to 'auth+acct'.  Please specify 'type = auth' or 'type = acct', etc.", p);
-		talloc_free(home);
-		goto error;
-	}
-
-#ifdef COA_TUNNEL
+#ifdef WITH_COA_TUNNEL
 	if (home->recv_coa) {
 		fr_strerror_printf("Dynamic home_server '%s' cannot receive CoA requests'", p);
 		talloc_free(home);
@@ -3189,8 +3234,88 @@ int home_server_afrom_file(char const *filename)
 	}
 #endif
 
+	old = home_server_byname(home->name, home->type);
+	if (old) {
+		if (!old->dynamic) {
+			fr_strerror_printf("Cannot replace static home server %s with a dynamic one",
+					   home->name);
+			talloc_free(home);
+			goto error;
+		}
+
+#ifdef WITH_TLS
+		if (!old->tls) {
+			fr_strerror_printf("Cannot replace non-TLS home server %s with a dynamic one",
+					   home->name);
+			talloc_free(home);
+			goto error;
+		}
+#endif
+
+#if 0
+		/*
+		 *	The fr_socket_client_tcp() and fr_socket() functions may change the
+		 *	source IP, i.e. * -> 192.168...., due to issues like FreeBSD jails.
+		 */
+		if (memcmp(&old->src_ipaddr, &home->src_ipaddr, sizeof(home->src_ipaddr)) != 0) {
+			fr_strerror_printf("Cannot change source IP for dynamic home server %s.",
+					   home->name);
+			talloc_free(home);
+			goto error;
+		}
+#endif
+
+		if (old->ipaddr.af != home->ipaddr.af) {
+			fr_strerror_printf("Cannot change IP address families for dynamic home server %s.",
+					   home->name);
+			talloc_free(home);
+			goto error;
+		}
+
+		/*
+		 *	No other thread is writing to it, as we're running in the master thread.  So this
+		 *	memcpy is safe.
+		 *
+		 *	@todo - extend the lifetime?
+		 */
+		if (memcmp(&old->ipaddr, &home->ipaddr, sizeof(home->ipaddr)) == 0) {
+			talloc_free(home);
+			return 0;
+		}
+
+		/*
+		 *	Change the destination IP to the new one.
+		 *
+		 *	This isn't thread-safe.  :(
+		 */
+		switch (old->ipaddr.af) {
+		case AF_INET:
+			old->ipaddr.ipaddr.ip4addr.s_addr = home->ipaddr.ipaddr.ip4addr.s_addr;
+			break;
+
+		case AF_INET6:
+			memcpy(&old->ipaddr.ipaddr.ip6addr.s6_addr,
+			       &home->ipaddr.ipaddr.ip6addr.s6_addr,
+			       sizeof(old->ipaddr.ipaddr.ip6addr.s6_addr));
+			break;
+
+		default:
+			fr_strerror_printf("Bad address family");
+			talloc_free(home);
+			return -1;
+		}
+
+		talloc_free(home);
+		return 0;
+	}
+
+	/*
+	 *	@todo - find the original one.  If it already exists,
+	 *	just change the IP address?
+	 */
+
 	if (!realm_home_server_add(home)) {
-		fr_strerror_printf("Failed adding home_server to the internal data structures");
+		fr_strerror_printf("Failed adding dynamic server");
 		talloc_free(home);
 		goto error;
 	}
@@ -3198,7 +3323,7 @@ int home_server_afrom_file(char const *filename)
 	return 0;
 }
 
-int home_server_delete(char const *name, char const *type_name)
+int home_server_delete_byname(char const *name, char const *type_name)
 {
 	home_server_t *home;
 	int type;
@@ -3228,13 +3353,24 @@ int home_server_delete(char const *name, char const *type_name)
 		return -1;
 	}
 
+	return home_server_delete(home);
+}
+
+int home_server_delete(home_server_t *home)
+{
 	if (!home->dynamic) {
-		fr_strerror_printf("Cannot delete static home_server %s", p);
+		fr_strerror_printf("Cannot delete static home_server %s", home->name);
 		return -1;
 	}
 
+#ifdef WITH_TLS
+	if (rbtree_num_elements(home->listeners) > 0) {
+		fr_strerror_printf("Cannot delete dynaic home_server %s - it still has open sockets", home->name);
+		return -1;
+	}
+#endif
+
 	(void) rbtree_deletebydata(home_servers_byname, home);
-	(void) rbtree_deletebydata(home_servers_byaddr, home);
 #ifdef WITH_STATS
 	(void) rbtree_deletebydata(home_servers_bynumber, home);
 #endif
