@@ -1,7 +1,7 @@
 /*
  * stats.c	Internal statistics handling.
  *
- * Version:	$Id: c1e4ec98fec5f8fdaffab2339643c3adce37df30 $
+ * Version:	$Id: 9c7d8f81b24003f027facbf91cd2c56e4aba4a98 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2008  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: c1e4ec98fec5f8fdaffab2339643c3adce37df30 $")
+RCSID("$Id: 9c7d8f81b24003f027facbf91cd2c56e4aba4a98 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -90,44 +90,66 @@ static void stats_time(fr_stats_t *stats, struct timeval *start,
 
 void request_stats_final(REQUEST *request)
 {
-	if (request->master_state == REQUEST_COUNTED) return;
+	rad_listen_t *listener;
+	RADCLIENT *client;
 
-	if (!request->listener) return;
-	if (!request->client) return;
+	if ((request->options & RAD_REQUEST_OPTION_STATS) != 0) return;
 
-	if ((request->listener->type != RAD_LISTEN_NONE) &&
-#ifdef WITH_ACCOUNTING
-	    (request->listener->type != RAD_LISTEN_ACCT) &&
-#endif
-#ifdef WITH_COA
-	    (request->listener->type != RAD_LISTEN_COA) &&
-#endif
-	    (request->listener->type != RAD_LISTEN_AUTH)) return;
+	/*
+	 *	This packet was originated by the server, and not
+	 *	received from a client.  It's a status-server or home
+	 *	server "ping" packet.  So we ignore it for statistics
+	 *	purposes.
+	 */
+	if (!request->packet) return;
 
 	/* don't count statistic requests */
-	if (request->packet->code == PW_CODE_STATUS_SERVER)
+	if (request->packet->code == PW_CODE_STATUS_SERVER) {
 		return;
+	}
+
+	listener = request->listener;
+	if (listener) switch (listener->type) {
+		case RAD_LISTEN_NONE:
+#ifdef WITH_ACCOUNTING
+		case RAD_LISTEN_ACCT:
+#endif
+#ifdef WITH_COA
+		case RAD_LISTEN_COA:
+#endif
+		case RAD_LISTEN_AUTH:
+			break;
+
+		default:
+			return;
+	}
+
+	/*
+	 *	Deal with TCP / TLS issues.  The statistics are kept in the parent socket.
+	 */
+	if (listener && listener->parent) listener = listener->parent;
+	client = request->client;
 
 #undef INC_AUTH
-#define INC_AUTH(_x) radius_auth_stats._x++;request->listener->stats._x++;request->client->auth._x++;
+#define INC_AUTH(_x) radius_auth_stats._x++;if (listener) listener->stats._x++;if (client) client->auth._x++;
 
 #undef INC_ACCT
 #ifdef WITH_ACCOUNTING
-#define INC_ACCT(_x) radius_acct_stats._x++;request->listener->stats._x++;request->client->acct._x++
+#define INC_ACCT(_x) radius_acct_stats._x++;if (listener) listener->stats._x++;if (client) client->acct._x++
 #else
 #define INC_ACCT(_x)
 #endif
 
 #undef INC_COA
 #ifdef WITH_COA
-#define INC_COA(_x) radius_coa_stats._x++;request->listener->stats._x++;request->client->coa._x++
+#define INC_COA(_x) radius_coa_stats._x++;if (listener) listener->stats._x++;if (client) client->coa._x++
 #else
 #define INC_COA(_x)
 #endif
 
 #undef INC_DSC
 #ifdef WITH_DSC
-#define INC_DSC(_x) radius_dsc_stats._x++;request->listener->stats._x++;request->client->dsc._x++
+#define INC_DSC(_x) radius_dsc_stats._x++;if (listener) listener->stats._x++;if (client) client->dsc._x++
 #else
 #define INC_DSC(_x)
 #endif
@@ -140,7 +162,7 @@ void request_stats_final(REQUEST *request)
 	 *	deleted, because only the main server thread calls
 	 *	this function, which makes it thread-safe.
 	 */
-	if (request->reply && (request->packet->code != PW_CODE_STATUS_SERVER)) switch (request->reply->code) {
+	if (request->reply) switch (request->reply->code) {
 	case PW_CODE_ACCESS_ACCEPT:
 		INC_AUTH(total_access_accepts);
 
@@ -156,7 +178,7 @@ void request_stats_final(REQUEST *request)
 		stats_time(&request->client->auth,
 			   &request->packet->timestamp,
 			   &request->reply->timestamp);
-		stats_time(&request->listener->stats,
+		stats_time(&listener->stats,
 			   &request->packet->timestamp,
 			   &request->reply->timestamp);
 		break;
@@ -268,7 +290,7 @@ void request_stats_final(REQUEST *request)
 	if (!request->proxy_reply) goto done;	/* simplifies formatting */
 
 #undef INC
-#define INC(_x) proxy_auth_stats._x += request->num_proxied_responses; request->home_server->stats._x += request->num_proxied_responses;
+#define INC(_x) proxy_auth_stats._x += request->num_proxied_responses;request->home_server->stats._x += request->num_proxied_responses;
 
 	switch (request->proxy_reply->code) {
 	case PW_CODE_ACCESS_ACCEPT:
@@ -341,9 +363,6 @@ void request_stats_final(REQUEST *request)
 
 
 	if (request->max_time) {
-		rad_listen_t *listener = request->listener;
-		RADCLIENT *client = request->client;
-
 		switch (request->packet->code) {
 		case PW_CODE_ACCESS_REQUEST:
 			FR_STATS_INC(auth, unresponsive_child);
@@ -369,7 +388,7 @@ void request_stats_final(REQUEST *request)
 		}
 	}
 
-	request->master_state = REQUEST_COUNTED;
+	request->options |= RAD_REQUEST_OPTION_STATS;
 }
 
 typedef struct fr_stats2vp {
@@ -518,8 +537,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Authentication.
 	 */
-	if (((flag->vp_integer & 0x01) != 0) &&
-	    ((flag->vp_integer & 0xc0) == 0)) {
+	if (((flag->vp_integer & 0x01) != 0) &&		/* auth */
+	    ((flag->vp_integer & 0xe0) == 0)) {		/* not client, server or home-server */
 		request_stats_addvp(request, authvp, &radius_auth_stats);
 	}
 
@@ -527,8 +546,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Accounting
 	 */
-	if (((flag->vp_integer & 0x02) != 0) &&
-	    ((flag->vp_integer & 0xc0) == 0)) {
+	if (((flag->vp_integer & 0x02) != 0) &&		/* accounting */
+	    ((flag->vp_integer & 0xe0) == 0)) {		/* not client, server or home-server */
 		request_stats_addvp(request, acctvp, &radius_acct_stats);
 	}
 #endif
@@ -537,8 +556,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Proxied authentication requests.
 	 */
-	if (((flag->vp_integer & 0x04) != 0) &&
-	    ((flag->vp_integer & 0x20) == 0)) {
+	if (((flag->vp_integer & 0x04) != 0) &&		/* proxy-auth */
+	    ((flag->vp_integer & 0x20) == 0)) {		/* not client */
 		request_stats_addvp(request, proxy_authvp, &proxy_auth_stats);
 	}
 
@@ -546,8 +565,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Proxied accounting requests.
 	 */
-	if (((flag->vp_integer & 0x08) != 0) &&
-	    ((flag->vp_integer & 0x20) == 0)) {
+	if (((flag->vp_integer & 0x08) != 0) &&		/* proxy-accounting */
+	    ((flag->vp_integer & 0x20) == 0)) {		/* not client */
 		request_stats_addvp(request, proxy_acctvp, &proxy_acct_stats);
 	}
 #endif
@@ -556,7 +575,7 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Internal server statistics
 	 */
-	if ((flag->vp_integer & 0x10) != 0) {
+	if ((flag->vp_integer & 0x10) != 0) {		/* internal */
 		vp = radius_pair_create(request->reply, &request->reply->vps,
 				       PW_FREERADIUS_STATS_START_TIME, VENDORPEC_FREERADIUS);
 		if (vp) vp->vp_date = start_time.tv_sec;
@@ -600,11 +619,11 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	For a particular client.
 	 */
-	if ((flag->vp_integer & 0x20) != 0) {
+	if ((flag->vp_integer & 0x20) != 0) { 		/* client */
 		fr_ipaddr_t ipaddr;
 		VALUE_PAIR *server_ip, *server_port = NULL;
 		RADCLIENT *client = NULL;
-		RADCLIENT_LIST *cl = NULL;
+		RADCLIENT_LIST *cl =  NULL;
 
 		/*
 		 *	See if we need to look up the client by server
@@ -619,6 +638,10 @@ void request_stats_reply(REQUEST *request)
 				ipaddr.ipaddr.ip4addr.s_addr = server_ip->vp_ipaddr;
 				cl = listener_find_client_list(&ipaddr, server_port->vp_integer, IPPROTO_UDP);
 
+#ifdef WITH_TCP
+				if (!cl) cl = listener_find_client_list(&ipaddr, server_port->vp_integer, IPPROTO_TCP);
+#endif
+
 				/*
 				 *	Not found: don't do anything
 				 */
@@ -634,6 +657,10 @@ void request_stats_reply(REQUEST *request)
 					ipaddr.ipaddr.ip6addr = server_ip->vp_ipv6addr;
 					cl = listener_find_client_list(&ipaddr, server_port->vp_integer, IPPROTO_UDP);
 
+#ifdef WITH_TCP
+					if (!cl) cl = listener_find_client_list(&ipaddr, server_port->vp_integer, IPPROTO_TCP);
+#endif
+
 					/*
 					 *	Not found: don't do anything
 					 */
@@ -757,8 +784,8 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	For a particular "listen" socket.
 	 */
-	if (((flag->vp_integer & 0x40) != 0) &&
-	    ((flag->vp_integer & 0x03) != 0)) {
+	if (((flag->vp_integer & 0x40) != 0) &&		/* server */
+	    ((flag->vp_integer & 0x03) != 0)) {		/* auth or accounting */
 		rad_listen_t *this;
 		VALUE_PAIR *server_ip, *server_port;
 		fr_ipaddr_t ipaddr;
@@ -787,6 +814,9 @@ void request_stats_reply(REQUEST *request)
 		 *	Not found: don't do anything
 		 */
 		this = listener_find_byipaddr(&ipaddr, server_port->vp_integer, IPPROTO_UDP);
+#ifdef WITH_TCP
+		if (!this) this = listener_find_byipaddr(&ipaddr, server_port->vp_integer, IPPROTO_TCP);
+#endif
 		if (!this) {
 			stats_error(request, "No such listener");
 			return;
@@ -797,7 +827,7 @@ void request_stats_reply(REQUEST *request)
 		fr_pair_add(&request->reply->vps,
 			fr_pair_copy(request->reply, server_port));
 
-		if ((flag->vp_integer & 0x01) != 0) {
+		if ((flag->vp_integer & 0x01) != 0) {	/* auth */
 			if ((request->listener->type == RAD_LISTEN_AUTH) ||
 			    (request->listener->type == RAD_LISTEN_NONE)) {
 				request_stats_addvp(request, authvp, &this->stats);
@@ -807,7 +837,7 @@ void request_stats_reply(REQUEST *request)
 		}
 
 #ifdef WITH_ACCOUNTING
-		if ((flag->vp_integer & 0x02) != 0) {
+		if ((flag->vp_integer & 0x02) != 0) {	/* accounting */
 			if ((request->listener->type == RAD_LISTEN_ACCT) ||
 			    (request->listener->type == RAD_LISTEN_NONE)) {
 				request_stats_addvp(request, acctvp, &this->stats);
@@ -822,11 +852,11 @@ void request_stats_reply(REQUEST *request)
 	/*
 	 *	Home servers.
 	 */
-	if (((flag->vp_integer & 0x80) != 0) &&
-	    ((flag->vp_integer & 0x03) != 0)) {
+	if (((flag->vp_integer & 0x80) != 0) &&		/* home-server */
+	    ((flag->vp_integer & 0x03) != 0)) {		/* auth or accounting */
 		home_server_t *home;
-		VALUE_PAIR *server_ip, *server_port;
-		fr_ipaddr_t ipaddr;
+		VALUE_PAIR *server_ip, *server_port, *server_src_ip;
+		fr_ipaddr_t ipaddr, src_ipaddr;
 
 		server_port = fr_pair_find_by_num(request->packet->vps, PW_FREERADIUS_STATS_SERVER_PORT, VENDORPEC_FREERADIUS, TAG_ANY);
 		if (!server_port) {
@@ -857,10 +887,33 @@ void request_stats_reply(REQUEST *request)
 			return;
 		}
 
+		memset(&src_ipaddr, 0, sizeof(src_ipaddr));
+		src_ipaddr.af = ipaddr.af;
+
+		if (ipaddr.af == AF_INET) {
+			server_src_ip = fr_pair_find_by_num(request->packet->vps, PW_FREERADIUS_STATS_SERVER_SRC_IP_ADDRESS, VENDORPEC_FREERADIUS, TAG_ANY);
+			if (server_src_ip) {
+				src_ipaddr.prefix = 32;
+				src_ipaddr.ipaddr.ip4addr.s_addr = server_src_ip->vp_ipaddr;
+			}
+#ifdef AF_INET6
+		} else if (ipaddr.af == AF_INET6) {
+			server_src_ip = fr_pair_find_by_num(request->packet->vps, PW_FREERADIUS_STATS_SERVER_SRC_IPV6_ADDRESS, VENDORPEC_FREERADIUS, TAG_ANY);
+			if (server_src_ip) {
+				src_ipaddr.af = AF_INET6;
+				src_ipaddr.ipaddr.ip6addr = server_src_ip->vp_ipv6addr;
+#endif	/* AF_INET6 */
+			}
+		}
+
+
 		/*
 		 *	Not found: don't do anything
 		 */
-		home = home_server_find(&ipaddr, server_port->vp_integer, IPPROTO_UDP);
+		home = home_server_find_bysrc(&ipaddr, server_port->vp_integer, IPPROTO_UDP, &src_ipaddr);
+#ifdef WITH_TCP
+		if (!home) home = home_server_find_bysrc(&ipaddr, server_port->vp_integer, IPPROTO_TCP, &src_ipaddr);
+#endif
 		if (!home) {
 			stats_error(request, "Failed to find home server IP");
 			return;
@@ -922,7 +975,7 @@ void request_stats_reply(REQUEST *request)
 				       PW_FREERADIUS_STATS_LAST_PACKET_SENT, VENDORPEC_FREERADIUS);
 		if (vp) vp->vp_date = home->last_packet_sent;
 
-		if ((flag->vp_integer & 0x01) != 0) {
+		if ((flag->vp_integer & 0x01) != 0) {	/* auth */
 			if (home->type == HOME_TYPE_AUTH) {
 				request_stats_addvp(request, proxy_authvp,
 						    &home->stats);
@@ -932,7 +985,7 @@ void request_stats_reply(REQUEST *request)
 		}
 
 #ifdef WITH_ACCOUNTING
-		if ((flag->vp_integer & 0x02) != 0) {
+		if ((flag->vp_integer & 0x02) != 0) {	/* accounting */
 			if (home->type == HOME_TYPE_ACCT) {
 				request_stats_addvp(request, proxy_acctvp,
 						    &home->stats);
@@ -978,14 +1031,14 @@ void radius_stats_ema(fr_stats_ema_t *ema,
 	}
 
 
-	tdiff = start->tv_sec;
-	tdiff -= end->tv_sec;
+	tdiff = end->tv_sec;
+	tdiff -= start->tv_sec;
 
 	micro = (int) tdiff;
 	if (micro > 40) micro = 40; /* don't overflow 32-bit ints */
 	micro *= USEC;
-	micro += start->tv_usec;
-	micro -= end->tv_usec;
+	micro += end->tv_usec;
+	micro -= start->tv_usec;
 
 	micro *= EMA_SCALE;
 
