@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: 5730b5e8448e77a8c0bd8b37195f2070581f8b57 $
+ * Version:	$Id: 2075ba12ec742b3d25974fee00a86fc8b222d857 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 5730b5e8448e77a8c0bd8b37195f2070581f8b57 $")
+RCSID("$Id: 2075ba12ec742b3d25974fee00a86fc8b222d857 $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include <freeradius-devel/radiusd.h>
@@ -136,6 +136,16 @@ typedef struct fr_pps_t {
 } fr_pps_t;
 #endif
 
+/*
+ *	In process.c, but no one else should be calling these
+ *	functions.
+ */
+extern void request_free(REQUEST *request);
+extern void request_done(REQUEST *request, int original);
+
+#ifndef HAVE_STDATOMIC_H
+static int request_fifo_discard(int priority, bool lower, time_t now);
+#endif
 
 /*
  *	A data structure to manage the thread pool.  There's no real
@@ -362,49 +372,63 @@ static void reap_children(void)
 int request_enqueue(REQUEST *request)
 {
 	bool managed = false;
+	time_t now;
 
 	rad_assert(pool_initialized == true);
 
+	/*
+	 *	Proxied packets get requeued when they time out, or
+	 *	when they get a reply.  We want to use "now" as the
+	 *	time, and not the time 5-10 seconds in the past when
+	 *	we originally received the request.
+	 */
+	if (request->priority == RAD_LISTEN_PROXY) {
+		now = time(NULL);
+	} else {
+		now = request->timestamp;
+	}
+
 	/*
 	 *	If we haven't checked the number of child threads
 	 *	in a while, OR if the thread pool appears to be full,
 	 *	go manage it.
 	 */
-	if (last_cleaned < request->timestamp) {
-		thread_pool_manage(request->timestamp);
+	if (last_cleaned < now) {
+		thread_pool_manage(now);
 		managed = true;
 	}
 
+	/*
+	 *	Update the request state.
+	 */
+	request->component = "<core>";
+	request->module = "<queue>";
+	request->child_state = REQUEST_QUEUED;
+
 #ifdef HAVE_STDATOMIC_H
 	if (!managed) {
 		uint32_t num;
 
 		num = load(thread_pool.active_threads);
 		if (num == thread_pool.total_threads) {
-			thread_pool_manage(request->timestamp);
+			thread_pool_manage(now);
 			managed = true;
 		}
 
 		if (!managed) {
 			num = load(thread_pool.exited_threads);
 			if (num > 0) {
-				thread_pool_manage(request->timestamp);
+				thread_pool_manage(now);
 			}
 		}
 	}
 
-	/*
-	 *	Use atomic queues where possible.  They're substantially faster than mutexes.
-	 */
-	request->component = "<core>";
-	request->module = "<queue>";
-	request->child_state = REQUEST_QUEUED;
-
 	/*
 	 *	Push the request onto the appropriate fifo for that
 	 */
 	if (!fr_atomic_queue_push(thread_pool.queue[request->priority], request)) {
-		ERROR("!!! ERROR !!! Failed inserting request %d into the queue", request->number);
+		RATE_LIMIT(ERROR("Something is blocking the server.  There are too many packets in the queue, "
+				 "waiting to be processed.  Ignoring the new request."));
 		return 0;
 	}
 
@@ -413,7 +437,7 @@ int request_enqueue(REQUEST *request)
 	if (!managed && 
 	    ((thread_pool.active_threads == thread_pool.total_threads) ||
 	     (thread_pool.exited_threads > 0))) {
-		thread_pool_manage(request->timestamp);
+		thread_pool_manage(now);
 	}
 
 	pthread_mutex_lock(&thread_pool.queue_mutex);
@@ -421,7 +445,7 @@ int request_enqueue(REQUEST *request)
 #ifdef WITH_STATS
 #ifdef WITH_ACCOUNTING
 	if (thread_pool.auto_limit_acct) {
-		struct timeval now;
+		struct timeval when;
 
 		/*
 		 *	Throw away accounting requests if we're too
@@ -481,7 +505,7 @@ int request_enqueue(REQUEST *request)
 			}
 		}
 
-		gettimeofday(&now, NULL);
+		gettimeofday(&when, NULL);
 
 		/*
 		 *	Calculate the instantaneous arrival rate into
@@ -490,7 +514,7 @@ int request_enqueue(REQUEST *request)
 		thread_pool.pps_in.pps = rad_pps(&thread_pool.pps_in.pps_old,
 						 &thread_pool.pps_in.pps_now,
 						 &thread_pool.pps_in.time_old,
-						 &now);
+						 &when);
 
 		thread_pool.pps_in.pps_now++;
 	}
@@ -499,27 +523,28 @@ int request_enqueue(REQUEST *request)
 
 	thread_pool.request_count++;
 
-	if (thread_pool.num_queued >= thread_pool.max_queue_size) {
+	/*
+	 *	If there are too many packets _overall_, OR the
+	 *	destinatio fifo is full, then try to delete a lower
+	 *	priority one.
+	 */
+	if (((thread_pool.num_queued >= thread_pool.max_queue_size) &&
+	     (request_fifo_discard(request->priority, true, now) == 0)) ||
+	    (fr_fifo_full(thread_pool.fifo[request->priority]) &&
+	     (request_fifo_discard(request->priority, false, now) == 0))) {
 		pthread_mutex_unlock(&thread_pool.queue_mutex);
-
-		/*
-		 *	Mark the request as done.
-		 */
 		RATE_LIMIT(ERROR("Something is blocking the server.  There are %d packets in the queue, "
 				 "waiting to be processed.  Ignoring the new request.", thread_pool.num_queued));
 		return 0;
 	}
 
-	request->component = "<core>";
-	request->module = "<queue>";
-	request->child_state = REQUEST_QUEUED;
-
 	/*
-	 *	Push the request onto the appropriate fifo for that
+	 *	Push the request onto the appropriate fifo for that priority.
 	 */
 	if (!fr_fifo_push(thread_pool.fifo[request->priority], request)) {
 		pthread_mutex_unlock(&thread_pool.queue_mutex);
-		ERROR("!!! ERROR !!! Failed inserting request %d into the queue", request->number);
+		RATE_LIMIT(ERROR("Something is blocking the server.  There are too many packets in the queue, "
+				 "waiting to be processed.  Ignoring the new request."));
 		return 0;
 	}
 
@@ -542,6 +567,79 @@ int request_enqueue(REQUEST *request)
 	return 1;
 }
 
+#ifndef HAVE_STDATOMIC_H
+/*
+ *	Try to free up requests by discarding requests of lower priority.
+ */
+static int request_fifo_discard(int priority, bool lower, time_t now)
+{
+	int i, rcode;
+	REQUEST *request;
+
+	if (lower) {
+		for (i = NUM_FIFOS - 1; i < priority; i--) {
+			request = fr_fifo_pop(thread_pool.fifo[i]);
+			if (!request) continue;
+
+			fr_assert(request->child_state == REQUEST_QUEUED);
+			request->child_state = REQUEST_DONE;
+			request_done(request, FR_ACTION_DONE);
+			return 1;
+		}
+
+		/*
+		 *	We didn't discard a lower priority entry.
+		 *	Maybe we need to discard one of the current
+		 *	priority, which has been in the queue for a
+		 *	while.
+		 */
+	}
+
+	/*
+	 *	The time stamp for proxied requests may be 5-10
+	 *	seconds in the past, because the home server hasn't
+	 *	responded.  We therefore can't discard "old" requests,
+	 *	as they have just received a reply, or they have just
+	 *	timed out.
+	 */
+	if (priority <= RAD_LISTEN_PROXY) return 0;
+
+	/*
+	 *	Peek at the first entry in the fifo.  Note that there
+	 *	is not always a first entry.  This is because we're
+	 *	called if there are too many _total_ requests.
+	 */
+	rcode = 0;
+
+retry:
+	request = fr_fifo_peek(thread_pool.fifo[priority]);
+	if (!request) return rcode;
+
+	/*
+	 *	This request expires in the future.  We can't do anything.
+	 */
+	if ((request->timestamp + main_config.max_request_time) > now) return rcode;
+
+	request = fr_fifo_pop(thread_pool.fifo[priority]);
+	rad_assert(request != NULL);
+	VERIFY_REQUEST(request);
+
+	request->child_state = REQUEST_DONE;
+	if (request->master_state == REQUEST_TO_FREE) {
+		request_free(request);
+	} else {
+		request_done(request, REQUEST_DONE);
+	}
+	thread_pool.num_queued--;
+
+	/*
+	 *	We might as well delete as many old requests as possible.
+	 */
+	rcode = 1;
+	goto retry;
+}
+#endif
+
 /*
  *	Remove a request from the queue.
  */
@@ -575,8 +673,32 @@ retry:
 
 		/*
 		 *	This entry was marked to be stopped.  Acknowledge it.
+		 *
+		 *	If we own the request, we delete it. Otherwise
+		 *	we run the "done" callback now, which will
+		 *	stop timers, remove it from the request hash,
+		 *	update listener counts, etc.
+		 *
+		 *	Running request_done() here means that old
+		 *	requests are cleaned up immediately, which
+		 *	frees up more resources for new requests.  It
+		 *	also means that we don't need to rely on
+		 *	timers to free up the old requests, as those
+		 *	timers will run much much later.
+		 *
+		 *	Catching this corner case doesn't change the
+		 *	normal operation of the server.  Most requests
+		 *	should NOT be marked "stop processing" when
+		 *	they're in the queue.  This situation
+		 *	generally happens when the server is blocked,
+		 *	due to a slow back-end database.
 		 */
 		request->child_state = REQUEST_DONE;
+		if (request->master_state == REQUEST_TO_FREE) {
+			request_free(request);
+		} else {
+			request_done(request, REQUEST_DONE);
+		}
 	}
 
 	/*
@@ -631,7 +753,13 @@ retry:
 		request = fr_fifo_pop(thread_pool.fifo[i]);
 		rad_assert(request != NULL);
 		VERIFY_REQUEST(request);
+
 		request->child_state = REQUEST_DONE;
+		if (request->master_state == REQUEST_TO_FREE) {
+			request_free(request);
+		} else {
+			request_done(request, REQUEST_DONE);
+		}
 		thread_pool.num_queued--;
 	}
 
@@ -663,6 +791,7 @@ retry:
 
 	rad_assert(*prequest != NULL);
 	rad_assert(request->magic == REQUEST_MAGIC);
+	rad_assert(request->child_state == REQUEST_QUEUED);
 
 	request->component = "<core>";
 	request->module = "";
@@ -1196,10 +1325,30 @@ int thread_pool_init(CONF_SECTION *cs, bool *spawn_flag)
 }
 DIAG_ON(deprecated-declarations)
 
-/*
- *	Stop all threads in the pool.
- */
 void thread_pool_stop(void)
+{
+	int i, total_threads;
+
+	if (!pool_initialized) return;
+
+	/*
+	 *	Set pool stop flag.
+	 */
+	thread_pool.stop_flag = true;
+
+	/*
+	 *	Wakeup all threads to make them see stop flag.
+	 */
+	total_threads = thread_pool.total_threads;
+	for (i = 0; i != total_threads; i++) {
+		sem_post(&thread_pool.semaphore);
+	}
+}
+
+/*
+ *	Free all thread-related information
+ */
+void thread_pool_free(void)
 {
 #ifndef WITH_GCD
 	int i;
@@ -1231,6 +1380,19 @@ void thread_pool_stop(void)
 		delete_thread(handle);
 	}
 
+	/*
+	 *	Free any requests which were blocked in the queue, but
+	 *	only if we're checking that no memory leaked.
+	 */
+	if (main_config.memory_report) {
+		REQUEST *request;
+
+		while (request_dequeue(&request) == 1) {
+			request->child_state = REQUEST_DONE;
+			request_free(request);
+		}
+	}
+
 	for (i = 0; i < NUM_FIFOS; i++) {
 #ifdef HAVE_STDATOMIC_H
 		fr_atomic_queue_free(&thread_pool.queue[i]);
