@@ -1,7 +1,7 @@
 /*
  * rlm_eap.h    Local Header file.
  *
- * Version:     $Id: 0b9311cd83a32ea741b51cb656017179772b3f73 $
+ * Version:     $Id: 4d5e1fa2694d7c8b211a1f5595ea14e6571d09b6 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #ifndef _RLM_EAP_H
 #define _RLM_EAP_H
 
-RCSIDH(rlm_eap_h, "$Id: 0b9311cd83a32ea741b51cb656017179772b3f73 $")
+RCSIDH(rlm_eap_h, "$Id: 4d5e1fa2694d7c8b211a1f5595ea14e6571d09b6 $")
 
 #include <freeradius-devel/modpriv.h>
 #include "eap.h"
@@ -63,14 +63,19 @@ typedef struct rlm_eap {
 
 	bool		ignore_unknown_types;
 	bool		mod_accounting_username_bug;
+	bool		allow_empty_identities;
 
 	uint32_t	max_sessions;
 
+	char const	*dedup_key;
+
 #ifdef HAVE_PTHREAD_H
 	pthread_mutex_t	session_mutex;
 	pthread_mutex_t	handler_mutex;
 #endif
 
+	rbtree_t	*dedup_tree;
+
 	char const	*xlat_name; /* no xlat's yet */
 	fr_randctx	rand_pool;
 } rlm_eap_t;
@@ -102,7 +107,7 @@ eap_handler_t 	*eap_handler(rlm_eap_t *inst, eap_packet_raw_t **eap_msg, REQUEST
 
 /* Memory Management */
 EAP_DS      	*eap_ds_alloc(eap_handler_t *handler);
-eap_handler_t 	*eap_handler_alloc(rlm_eap_t *inst);
+eap_handler_t 	*eap_handler_alloc(rlm_eap_t *inst, REQUEST *request);
 void	    	eap_ds_free(EAP_DS **eap_ds);
 int 	    	eaplist_add(rlm_eap_t *inst, eap_handler_t *handler) CC_HINT(nonnull);
 eap_handler_t 	*eaplist_find(rlm_eap_t *inst, REQUEST *request, eap_packet_raw_t *eap_packet);
