@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 9cf99d832ebd7e87d49398c29ee26bb857a9c092 $
+ * $Id: a2476e8f1fa858177a2952384dc19dd9d58081b6 $
  * @file rlm_json.c
  * @brief Parses JSON responses
  *
@@ -26,7 +26,7 @@
  * @copyright 2015,2021 Network RADIUS SARL (legal@networkradius.com)
  * @copyright 2015 The FreeRADIUS Server Project
  */
-RCSID("$Id: 9cf99d832ebd7e87d49398c29ee26bb857a9c092 $")
+RCSID("$Id: a2476e8f1fa858177a2952384dc19dd9d58081b6 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -81,7 +81,7 @@ static CONF_PARSER const module_config[] = {
  * @param outlen space available for the output
  * @return length of output generated
  */
-static ssize_t json_encode_xlat(UNUSED void * instance, REQUEST *request, char const *fmt,
+static ssize_t json_encode_xlat(void * instance, REQUEST *request, char const *fmt,
 				      char *out, size_t outlen)
 {
 	rlm_json_t const	*inst = instance;
