@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: aaa43ab73d7d47b87cb1cad7b969fae90a0d32bc $
+ * $Id: 4f0f68d7f3416ff362de68b87c09b05b5981826b $
  * @file rlm_python3.c
  * @brief Translates requests between the server an a python interpreter.
  *
@@ -25,7 +25,7 @@
  * @copyright 2002  Miguel A.L. Paraz <mparaz@mparaz.com>
  * @copyright 2002  Imperium Technology, Inc.
  */
-RCSID("$Id: aaa43ab73d7d47b87cb1cad7b969fae90a0d32bc $")
+RCSID("$Id: 4f0f68d7f3416ff362de68b87c09b05b5981826b $")
 
 #define LOG_PREFIX "rlm_python3 - "
 
@@ -56,11 +56,11 @@ RCSID("$Id: aaa43ab73d7d47b87cb1cad7b969fae90a0d32bc $")
 static uint32_t		python_instances = 0;
 static void		*python_dlhandle;
 
-static PyThreadState	*main_interpreter;	//!< Main interpreter (cext safe)
-static PyObject		*main_module;		//!< Pthon configuration dictionary.
+static PyThreadState	*main_interpreter = NULL;	//!< Main interpreter (cext safe)
+static PyObject		*main_module = NULL;		//!< Pthon configuration dictionary.
 
-static rlm_python_t *current_inst;		//!< Needed to pass parameter to PyInit_radiusd
-static CONF_SECTION *current_conf;		//!< Needed to pass parameter to PyInit_radiusd
+static rlm_python_t *current_inst = NULL;		//!< Needed to pass parameter to PyInit_radiusd
+static CONF_SECTION *current_conf = NULL;		//!< Needed to pass parameter to PyInit_radiusd
 
 /*
  *	A mapping of configuration file names to internal variables.
@@ -90,6 +90,7 @@ static CONF_PARSER module_config[] = {
 #undef A
 
         { "python_path", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_python_t, python_path), NULL },
+	{ "python_path_mode", FR_CONF_OFFSET(PW_TYPE_STRING, rlm_python_t, python_path_mode_str), "append" },
         { "cext_compat", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, cext_compat), "yes" },
         { "pass_all_vps", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, pass_all_vps), "no" },
         { "pass_all_vps_dict", FR_CONF_OFFSET(PW_TYPE_BOOLEAN, rlm_python_t, pass_all_vps_dict), "no" },
@@ -133,6 +134,19 @@ static struct {
 	{ NULL, 0 },
 };
 
+typedef enum {
+	PYTHON_PATH_MODE_APPEND,
+	PYTHON_PATH_MODE_PREPEND,
+	PYTHON_PATH_MODE_OVERWRITE
+} py_path_mode;
+
+FR_NAME_NUMBER const python_path_mode[] = {
+	{ "append",	PYTHON_PATH_MODE_APPEND    },
+	{ "prepend",	PYTHON_PATH_MODE_PREPEND   },
+	{ "overwrite",	PYTHON_PATH_MODE_OVERWRITE },
+	{ NULL, -1 }
+};
+
 /*
  *	This allows us to initialise PyThreadState on a per thread basis
  */
@@ -204,12 +218,12 @@ static void python_error_log(void)
 		return;
 	}
 
-	if (((pStr1 = PyObject_Str(pExcType)) != NULL) && 
+	if (((pStr1 = PyObject_Str(pExcType)) != NULL) &&
 	    ((pStr2 = PyObject_Str(pExcValue)) != NULL)) {
 		ERROR("%s:%d, Exception type: %s, Exception value: %s", __func__, __LINE__, PyUnicode_AsUTF8(pStr1), PyUnicode_AsUTF8(pStr2));
 		Py_DECREF(pStr1);
 		Py_DECREF(pStr2);
-	} 
+	}
 
 	if (pExcTraceback) {
 		PyObject *pRepr = PyObject_Repr(pExcTraceback);
@@ -387,8 +401,9 @@ static int mod_populate_vptuple(PyObject *pPair, VALUE_PAIR *vp)
 		ERROR("%s:%d, vp->da->name: %s", __func__, __LINE__, vp->da->name);
 		if (PyErr_Occurred()) {
 			python_error_log();
+			PyErr_Clear();
 		}
-		
+
 		return -1;
 	}
 
@@ -567,6 +582,7 @@ static rlm_rcode_t do_python_single(REQUEST *request, PyObject *pFunc, char cons
 		ERROR("%s:%d, %s - pRet is NULL", __func__, __LINE__, funcname);
 		if (PyErr_Occurred()) {
 			python_error_log();
+			PyErr_Clear();
 		}
 		ret = RLM_MODULE_FAIL;
 		goto finish;
@@ -674,17 +690,22 @@ finish:
 	if (ret == RLM_MODULE_FAIL) {
 		ERROR("%s:%d, %s - RLM_MODULE_FAIL", __func__, __LINE__, funcname);
 	}
+
+	if (PyErr_Occurred()) {
+		ERROR("Unhandled Python exception (see below); clearing.");
+		python_error_log();
+		PyErr_Clear();
+	}
+
 	return ret;
 }
 
 static void python_interpreter_free(PyThreadState *interp)
 {
-DIAG_OFF(deprecated-declarations)
-	PyEval_AcquireLock();
+	PyEval_RestoreThread(interp);
 	PyThreadState_Swap(interp);
 	Py_EndInterpreter(interp);
-	PyEval_ReleaseLock();
-DIAG_ON(deprecated-declarations)
+	PyEval_SaveThread();
 }
 
 /** Destroy a thread state
@@ -1029,7 +1050,12 @@ static void *dlopen_libpython(int flags)
 /*
  * creates a module "radiusd"
  */
+#if PY_MINOR_VERSION >= 9
+PyMODINIT_FUNC PyInit_radiusd(void);
+PyMODINIT_FUNC PyInit_radiusd(void)
+#else
 static PyMODINIT_FUNC PyInit_radiusd(void)
+#endif
 {
 	CONF_SECTION *cs;
 	/*
@@ -1094,6 +1120,8 @@ static PyMODINIT_FUNC PyInit_radiusd(void)
  */
 static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 {
+	bool locked = false;
+
 	/*
 	 * prepare radiusd module to be loaded
 	 */
@@ -1115,7 +1143,33 @@ static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 		python_dlhandle = dlopen_libpython(RTLD_NOW | RTLD_GLOBAL);
 		if (!python_dlhandle) WARN("Failed loading libpython symbols into global symbol table");
 
-#if PY_VERSION_HEX >= 0x03050000
+#if PY_VERSION_HEX > 0x030a0000
+		{
+			PyStatus status;
+			PyConfig config;
+			wchar_t  *name;
+
+			/*
+			 *	Isolated config: Don't override signal handlers - noop on subs calls
+			 */
+			PyConfig_InitIsolatedConfig(&config);
+
+			MEM(name = Py_DecodeLocale(main_config.name, NULL));
+			status = PyConfig_SetString(&config, &config.program_name, name);
+			PyMem_RawFree(name);
+			if (PyStatus_Exception(status)) {
+				PyConfig_Clear(&config);
+				return -1;
+			}
+
+			status = Py_InitializeFromConfig(&config);
+			if (PyStatus_Exception(status)) {
+				PyConfig_Clear(&config);
+				return -1;
+			}
+			PyConfig_Clear(&config);
+		}
+#elif PY_VERSION_HEX >= 0x03050000
 		{
 			wchar_t  *name;
 
@@ -1132,11 +1186,18 @@ static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 		}
 #endif
 
+#if PY_VERSION_HEX <= 0x030a0000
 		Py_InitializeEx(0);			/* Don't override signal handlers - noop on subs calls */
+#if PY_VERSION_HEX <= 0x03060000
 		PyEval_InitThreads(); 			/* This also grabs a lock (which we then need to release) */
+#endif
+#endif
 		main_interpreter = PyThreadState_Get();	/* Store reference to the main interpreter */
+		locked = true;
 	}
+#if PY_VERSION_HEX < 0x03090000
 	rad_assert(PyEval_ThreadsInitialized());
+#endif
 
 	/*
 	 *	Increment the reference counter
@@ -1153,6 +1214,7 @@ static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 		inst->sub_interpreter = main_interpreter;
 	}
 
+	if (!locked) PyEval_AcquireThread(inst->sub_interpreter);
 	PyThreadState_Swap(inst->sub_interpreter);
 
 	/*
@@ -1171,25 +1233,38 @@ static int python_interpreter_init(rlm_python_t *inst, CONF_SECTION *conf)
 		if (inst->python_path) {
 			char *p, *path;
 			PyObject *sys = PyImport_ImportModule("sys");
-			PyObject *sys_path = PyObject_GetAttrString(sys, "path");
+			PyObject *sys_path;
+			Py_ssize_t i = 0;
 
 			memcpy(&p, &inst->python_path, sizeof(path));
 
+			if (inst->python_path_mode == PYTHON_PATH_MODE_OVERWRITE) {
+				sys_path = PyList_New(0);
+			} else {
+				sys_path = PyObject_GetAttrString(sys, "path");
+			}
+
 			for (path = strtok(p, ":"); path != NULL; path = strtok(NULL, ":")) {
+#if PY_VERSION_HEX > 0x03000000
+				wchar_t *py_path;
+
 #if PY_VERSION_HEX > 0x03050000
-				wchar_t *py_path;
-
 				MEM(py_path = Py_DecodeLocale(path, NULL));
-				PyList_Append(sys_path, PyUnicode_FromWideChar(py_path, -1));
-				PyMem_RawFree(py_path);
-#elif PY_VERSION_HEX > 0x03000000
-				wchar_t *py_path;
-
+#else
 				MEM(py_path = _Py_char2wchar(path, NULL));
-				PyList_Append(sys_path, PyUnicode_FromWideChar(py_path, -1));
+#endif
+				if (inst->python_path_mode == PYTHON_PATH_MODE_PREPEND) {
+					PyList_Insert(sys_path, i++, PyUnicode_FromWideChar(py_path, -1));
+				} else {
+					PyList_Append(sys_path, PyUnicode_FromWideChar(py_path, -1));
+				}
 				PyMem_RawFree(py_path);
 #else
-				PyList_Append(sys_path, PyLong_FromString(path));
+				if (inst->python_path_mode == PYTHON_PATH_PREPEND) {
+					PyList_Insert(sys_path, i++, PyLong_FromString(path));
+				} ekse {
+					PyList_Append(sys_path, PyLong_FromString(path));
+				}
 #endif
 			}
 
@@ -1228,6 +1303,13 @@ static int mod_instantiate(CONF_SECTION *conf, void *instance)
 	inst->name = cf_section_name2(conf);
 	if (!inst->name) inst->name = cf_section_name1(conf);
 
+	inst->python_path_mode = fr_str2int(python_path_mode, inst->python_path_mode_str, -1);
+	if (inst->python_path_mode < 0) {
+		cf_log_err_cs(conf, "Invalid 'python_path_mode' value \"%s\", expected 'append', "
+			      "'prepend' or 'overwrite'", inst->python_path_mode_str);
+		return -1;
+	}
+
 	/*
 	 *	Load the python code required for this module instance
 	 */
@@ -1260,9 +1342,9 @@ static int mod_instantiate(CONF_SECTION *conf, void *instance)
 	PYTHON_FUNC_LOAD(detach);
 
 	/*
-	 *	Call the instantiate function only if the function and module is set.
+	 *	Call the instantiate function only if the function and module is set and we're not doing a config check.
 	 */
-	if (inst->instantiate.module_name && inst->instantiate.function_name) {
+	if (!check_config && inst->instantiate.module_name && inst->instantiate.function_name) {
 
 		code = do_python_single(NULL, inst->instantiate.function, "instantiate", inst->pass_all_vps, inst->pass_all_vps_dict);
 		if (code < 0) {
@@ -1287,7 +1369,7 @@ static int mod_detach(void *instance)
 	 */
 	PyEval_RestoreThread(inst->sub_interpreter);
 
-	if (inst->detach.function) ret = do_python_single(NULL, inst->detach.function, "detach", inst->pass_all_vps, inst->pass_all_vps_dict);
+	if (!check_config && inst->detach.function) ret = do_python_single(NULL, inst->detach.function, "detach", inst->pass_all_vps, inst->pass_all_vps_dict);
 
 #define PYTHON_FUNC_DESTROY(_x) python_function_destroy(&inst->_x)
 	PYTHON_FUNC_DESTROY(instantiate);
@@ -1327,7 +1409,7 @@ static int mod_detach(void *instance)
 	if (!inst->cext_compat) python_interpreter_free(inst->sub_interpreter);
 
 	if ((--python_instances) == 0) {
-		PyThreadState_Swap(main_interpreter); /* Swap to the main thread */
+		PyEval_RestoreThread(main_interpreter); /* Swap to the main thread */
 		Py_Finalize();
 		dlclose(python_dlhandle);
 	}
