@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: c708470f0e01013e6a5c591738327e18542d2a5a $
+ * $Id: c62edea675fd52d0a3eff33c9a707428918528ba $
  * @file trustrouter.c
  * @brief Integration with external trust router code
  *
@@ -320,7 +320,7 @@ static fr_tls_server_conf_t *construct_tls(TIDC_INSTANCE *inst,
 {
 	fr_tls_server_conf_t *tls;
 	unsigned char *key_buf = NULL;
-	ssize_t keylen;
+	ssize_t keylen = 0;
 	char *hexbuf = NULL;
 	DH *aaa_server_dh;
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -359,7 +359,7 @@ static fr_tls_server_conf_t *construct_tls(TIDC_INSTANCE *inst,
 
 	tls->cipher_list = talloc_strdup(tls, "aPSK");
 	tls->fragment_size = 4200;
-	tls->ctx = tls_init_ctx(tls, 1);
+	tls->ctx = tls_init_ctx(tls, 1, NULL, NULL);
 	if (!tls->ctx) goto error;
 
 	memset(key_buf, 0, keylen);
