@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 24638f09cd9f00b3a97d832854f5babc181d12c6 $
+ * $Id: 9d21243683894524f6e80096ecdda40a7fe3851f $
  * @file rlm_sql.c
  * @brief Implements SQL 'users' file, and SQL accounting.
  *
@@ -24,7 +24,7 @@
  * @copyright 2000  Mike Machado <mike@innercite.com>
  * @copyright 2000  Alan DeKok <aland@ox.org>
  */
-RCSID("$Id: 24638f09cd9f00b3a97d832854f5babc181d12c6 $")
+RCSID("$Id: 9d21243683894524f6e80096ecdda40a7fe3851f $")
 
 #include <ctype.h>
 
@@ -1605,7 +1605,7 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 	VALUE_PAIR		*vp;
 	int			ret;
 	fr_ipaddr_t		nas_addr;
-	uint32_t		nas_port = 0;
+	char const		*nas_port_id = NULL;
 
 	char 			*expanded = NULL;
 
@@ -1706,7 +1706,7 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 
 		num_rows = (inst->module->sql_num_fields)(handle, inst->config);
 		if (num_rows < 8) {
-			RDEBUG("Too few rows returned.  Please do not edit 'simul_verify_query'");
+			WARN("SELECT returned too few fields.  Please do not edit 'simul_verify_query'");
 			rcode = RLM_MODULE_FAIL;
 
 			goto finish;
@@ -1734,11 +1734,9 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 			}
 		}
 
-		if (row[4]) {
-			nas_port = atoi(row[4]);
-		}
+		nas_port_id = row[4];
 
-		check = rad_check_ts(&nas_addr, nas_port, row[2], row[1]);
+		check = rad_check_ts(&nas_addr, 0, nas_port_id, row[2], row[1]);
 		if (check == 0) {
 			/*
 			 *	Stale record - zap it.
@@ -1758,7 +1756,7 @@ static rlm_rcode_t mod_checksimul(void *instance, REQUEST * request)
 				}
 				if ((num_rows > 8) && row[8])
 					sess_time = atoi(row[8]);
-				session_zap(request, &nas_addr, nas_port,
+				session_zap(request, &nas_addr, 0, nas_port_id,
 					    row[2], row[1], framed_addr,
 					    proto, sess_time);
 			}
